# Created by Wayne Porter
# https://raw.githubusercontent.com/OllisGit/OctoPrint-DisplayLayerProgress/master/Cura-PostProcessor/InsertAtLayerChangeEnhancement.py
# https://community.octoprint.org/t/tft-percentage-progress-not-updatet-m73/57420/40?u=lethuer
# Locate in: [CuraDirectory]\share\cura\plugins\PostProcessingPlugin\scripts
# modified by lethuer 2024-03-30

from ..Script import Script

class InsertAtLayerChangeEnhancementForTotalLayer(Script):
    def __init__(self):
        super().__init__()

    def getSettingDataString(self):
        return """{
            "name": "Insert at layer change (enhancement for total layer)",
            "key": "InsertAtLayerChangeEnhancementForTotalLayer",
            "metadata": {},
            "version": 2,
            "settings":
            {
                "insert_location":
                {
                    "label": "When to insert",
                    "description": "Whether to insert code before or after layer change.",
                    "type": "enum",
                    "options": {"before": "Before", "after": "After"},
                    "default_value": "before"
                },
                "gcode_to_add":
                {
                    "label": "GCODE to insert.",
                    "description": "GCODE to add before or after layer change. You can add '[layer_num]' as a placeholder for the current layer number and [layer_count] as a placeholder of lotal layer count",
                    "type": "str",
                    "default_value": "M118 P0 A1 action:notification Layer Left [layer_num]/[layer_count]"
                }
            }
        }"""

    def execute(self, data):
        gcode_to_add = self.getSettingValueByKey("gcode_to_add") + "\n"
        max_layer = 0
        for layer in data:
            # Check that a layer is being printed
            lines = layer.split("\n")
            for line in lines:
                if line.startswith(";LAYER_COUNT:"):
                    max_layer = line
                    max_layer = max_layer.split(":")[1]
                    max_layer = int(max_layer) - 1
                    max_layer = str(max_layer)
                if ";LAYER:" in line:
                    index = data.index(layer)
                    layerNumber = line[7:]
                    newGcode_to_add = gcode_to_add.replace("[layer_num]", layerNumber).replace("[layer_count]", max_layer)
                    if self.getSettingValueByKey("insert_location") == "before":
                        layer = newGcode_to_add + layer
                    else:
                        layer = layer + newGcode_to_add

                    data[index] = layer
                    break
        return data